module TuringHS.Operator where

data Operator
  = -- | ">" Increment the data pointer (to point to the next cell to the right).
    Forward
  | -- | "<" Decrement the data pointer (to point to the next cell to the left).
    Backward
  | -- | "+" Increment (increase by one) the byte at the data pointer.
    Increment
  | -- | "-" Decrement (decrease by one) the byte at the data pointer.
    Decrement
  | -- | "." Output the byte at the data pointer.
    Put
  | -- | "," Accept one byte of input, storing its value in the byte at the data
    -- pointer.
    Get
  | -- | "[" If the byte at the data pointer is zero, then instead of moving the
    -- instruction pointer forward to the next command, jump it forward to the
    -- command after the matching ] command.
    JumpForward
  | -- | "]" If the byte at the data pointer is nonzero, then instead of moving
    -- the instruction pointer forward to the next command, jump it back to the
    -- command after the matching [ command.
    JumpBackward
  deriving (Eq, Show)

getOperator :: Char -> Maybe Operator
getOperator c = case c of
  '>' -> Just Forward
  '<' -> Just Backward
  '+' -> Just Increment
  '-' -> Just Decrement
  '.' -> Just Put
  ',' -> Just Get
  '[' -> Just JumpForward
  ']' -> Just JumpBackward
  _ -> Nothing
