module TuringHS.OperatorSpec where

import Data.Foldable
import Test.Hspec
import TuringHS.Operator

spec :: Spec
spec = do
  describe "getOperator" $ do
    describe "happy-path" $ do
      it "should get Forward from '>'" $ do
        getOperator '>' `shouldBe` Just Forward
      it "should get Backward from '<" $ do
        getOperator '<' `shouldBe` Just Backward
      it "should get Increment from '+'" $ do
        getOperator '+' `shouldBe` Just Increment
      it "should get Decrement from '-'" $ do
        getOperator '-' `shouldBe` Just Decrement
      it "should get Put from '.'" $ do
        getOperator '.' `shouldBe` Just Put
      it "should get Get from ','" $ do
        getOperator ',' `shouldBe` Just Get
      it "should get JumpForward from '['" $ do
        getOperator '[' `shouldBe` Just JumpForward
      it "should get JumpBackward from ']'" $ do
        getOperator ']' `shouldBe` Just JumpBackward

    describe "unhappy-path" $ do
      traverse_ makeBadExample badInput

badInput :: String
badInput = ['a' .. 'z'] ++ ['0' .. '9'] ++ ['@', '$', ' ', '\n', '\t']

makeBadExample :: Char -> SpecWith ()
makeBadExample input = do
  it ("should not get anything from " ++ show input) $ do
    getOperator input `shouldBe` Nothing
